:- module(hs_file,_,[assertions]).
:- use_module(library(atom2term)).
:- use_module(library(terms)).
:- use_module(library(iso_byte_char)).
:- use_module(library(lists)).
% :- use_module(library(write)).
:- use_module(library(system)).
:- use_module(library(file_utils)).

:- use_module(hs_config).
:- doc(author, "Christian Giménez").
:- doc(copyright, "
hs_file.pl
Copyright (C) 2012  Giménez, Christian N.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
Domingo 09 De Septiembre Del 2012    
").

:- doc(section, "A String Problem").
:- doc(summary, "There is a string problem, the @lib{atom2term} module have problems when you want to transform a string
	with '/' at the beggining and in the middle. Predicates like @pred{string2term/2} give a bad results and I don't know why.

So we split the URL into a list of folders using the '/' as separator, and then we transform them into terms. After that, we concat the atoms.

With the result of all this process, we have an URL as a term.

In other words: 

@begin{enumerate}
@item Split the string into a list of folder(and archive) names
@item Transform each string(folder names) of the list into an term
@item Concatenate each term into a complete URL again
@item that's it!
@end{enumerate} ").

%%%%%%%%%%%%%%%%%%%%
:- pred up_to_bar(Str, Word) # "True iff unifys @var{Word} with the string that is up to the first '/'.
For example:
@begin{verbatim}
	up_to_bar(\"/hi/word/hello/\",\"hi\").
@end{verbatim}
Returns true.

Note: It ignores the first \"/\".".

up_to_bar([47|Str], Word) :-
	up_to_bar2(Str,Word),!. %% Just ignore the first '/' once!

up_to_bar(Str,Word) :-
	up_to_bar2(Str,Word).

up_to_bar2([],[]):-!.
up_to_bar2([47|_], []) :- !.	

up_to_bar2([X|Rest], [X|Word]) :-
	up_to_bar2(Rest, Word).


%%%%%%%%%%%%%%%%%%%%
:- pred up_to_barB(Str, Word) # "This does the same as @pred{up_to_bar/2}, except that it will add the first and last '/' if the string has it:

For example:
@begin{verbatim}
	up_to_barB(\"/hi/word/hello/\",\"/hi/\").
	up_to_barB(\"hi/word/hello/\",\"hi/\").
	up_to_barB(\"hi\",\"hi\").
@end{verbatim}

Return true.".

up_to_barB([47|Str], [47|Word]) :-
	up_to_barB2(Str,Word),!. %% Just ignore the first '/' once, but add it to the result!

up_to_barB(Str,Word) :-
	up_to_barB2(Str,Word).

up_to_barB2([],[]):-!.
up_to_barB2([47|_], [47]) :- !.	

up_to_barB2([X|Rest], [X|Word]) :-
	up_to_barB2(Rest, Word).


:- pred split(String, List) # "Is true iff @var{List} unifies with the list of string that is produced by dividing the @var{String} into parts using the '/' as separator

For example:
@begin{verbatim}
	up_to_bar(\"/hi/word/hello/\",[\"hi\",\"word\",\"hello\"]).
@end{verbatim}

Returns true.".

split([], []) :- !.
split(Str, [Word|L]) :-	
	up_to_bar(Str,Word), %% What is the String up to the "/"?
	up_to_barB(Str,Word2), %% Retrieve the first string up to the, but add '/' if it has.
	append(Word2, Sig, Str), %% What is the rest excluding the Word string?
	split(Sig, L). %% Split the rest of the string.
	


%%%%%%%%%%%%%%%%%%%%
:- doc(subsection, "Joining all Toghether").

:- pred lstStrs2lstTerms(Strs, Terms) # "Transform a list of strings into a list of terms.".

lstStrs2lstTerms([], []) :- !.
lstStrs2lstTerms([X|Strs], [Y|Terms]) :-
	string2term(X,Y),
	lstStrs2lstTerms(Strs, Terms).

:- pred join_terms(Terms, Out) # "True if @var{Out} Unifies with all the terms in the list @var{Terms} concatenated with '/' between.
For example:
@begin{verbatim}
	join_terms([hi, word, hello], 'hi/word/hello').
@end{verbatim}
Retuns true.".
join_terms(Terms, Out) :-
	add_bars([''| Terms],Lst),
	atom_concat(Lst, Out).
	
add_bars([X|[]], [X]) :- !.
add_bars([X|Lst], [X,'/'|LstBars]) :- 
	add_bars(Lst, LstBars).
	
	
:- doc(subsection, "Finale").

:- pred string2termB(Str, Term) # "Like @pred{string2term} but tolerate when there are '/' characters.".
string2termB(Str, Term) :-
	split(Str,Y),
	lstStrs2lstTerms(Y,Z),
	join_terms(Z,Term).
	

:- doc(section,"Using the File").
:- doc(summary, "Functions for opening and manipulating the file.").


:- pred index_type(+Type) # "Some predicates of the common index files used as index.".
index_type('index.pl').
index_type('index.html').


:- pred append_index(+RealFile1, -RealFile) # "Is true iff @var{RealFile} is an index path and the file exists".
append_index(RealFile1, RealFile) :-	
	index_type(Index),
	atom_concat([RealFile1,'/',Index], RealFile),
	file_exists(RealFile),!.

append_index(RealFile1, RealFile1) :- !.

:- pred open_file(+File, -Str) # "Open the file given by the path in the string @var{File}. If the file is in the home dir, and it can be opened, maintaint it opened.

Use @pred{close_file/0} for closing it and @pred{next_byte/1} for reading the next byte in the file.".
open_file(File, Str) :-
	home_dir(Path),
	append(Path, File, RealFileStr),
	% display_string(RealFileStr),nl,
	string2termB(RealFileStr, RealFile1),
	append_index(RealFile1,RealFile),
	% write(RealFile),nl,
	%% open(RealFile, read, Stream),
	%% set_input(Stream).
	file_to_string(RealFile, Str).

:- pred close_file # "Close the file stream.".
close_file :-
	current_input(Stream),
	close(Stream).


:- pred next_byte(Byte) # "True iff @var{Byte} is unified with the next Byte of the file. 
If the end of file is reached, @var{Byte} will be unified with the @tt{end_of_file} atom.".
next_byte(end_of_file) :-
	peek_byte(-1),!.

next_byte(B) :-
	get_char(B).